#include <L298.h>

L298 m = L298(2,3,4,7,5,6);

void setup() {
  Serial.begin(9600);
}

unsigned long uptime = millis();
void readline(char *line, int siz) {
  for (int i=0; i < siz; i++)
    line[i]=0;
  int head=0;
  while (1) {
    if (millis()-uptime > 100)
      m.stop();
    if (!Serial.available())
      continue;
    uptime = millis();
    line[head++] = Serial.read();
    if (!(head<siz) || line[head-1]== '\n') {
      return;
    }
  }
}

void loop() {
  char buff[64];
  readline(buff, 64);
  switch (buff[0]) {
    case 'A': //analog read
      rpcA(buff+1);
      break;
    case 'W': //write to IO
      rpcW(buff+1);
      break;
    case 'M': //move wheel
      rpcM(buff+1);
      break;
    case 'T': //turn ms
      rpcT(buff+1);
      break;
    case 'D': //distanse
      rpcD(buff+1);
      break;
    case 'S': //distanse
      rpcS(buff+1);
      break;
    case '\r':
      Serial.println("ok");
      break;
    default:
      return;
  }
}

void rpcA(char *arg) {
  int pin;
  sscanf(arg, "%d", &pin);
  Serial.println(analogRead(pin));
}

void rpcW(char *arg) {
  int pin, v;
  sscanf(arg, "%d%d", &pin, &v);
  digitalWrite(pin, v);
  Serial.println("ok");
}

void rpcM(char *arg) {
  int l, r;
  sscanf(arg, "%d%d", &l, &r);
  m.feed(0, l);
  m.feed(1, r);
  Serial.println("ok");
}


/*********************************************************************/
void rpcT(char *arg) {
  int ms;
  sscanf(arg, "%d", &ms);
  turn(ms, 0);
  Serial.println("ok");
}

#define TURN90 730
#define TURN180 1400
void turn(int ms, int dir) {
  if (dir) {
    m.feed(0, -255);
    m.feed(1, 255);
  } else {
    m.feed(0, 255);
    m.feed(1, -255);
  }
  delay(ms);
  m.stop();
}

void strait() {
  m.feed(0, 168);
  m.feed(1, 127);
}
/*********************************************************************/


/*********************************************************************/
int adc(int ch) {
  float m = 100, v;
  for (int i=0; i<10; i++) {
    v = analogRead(ch);
    if (m < v) {
      m = v;
    }
  }
  return m;
}

float dist(int ch) {
  return 4800.0/(adc(ch) - 20);
}

void rpcD(char *arg) {
  int ch;
  sscanf(arg, "%d", &ch);
  Serial.println(dist(ch));
}
/*********************************************************************/


void rpcS(char *arg) {
  Serial.println("ok");
  switch (arg[0]) {
    case '0':
      random_sweep();
      break;
    case '1':
      bfs_sweep();
      break;
  }
}

void random_sweep() {
  char dir=0;
  while (!Serial.available()) {
    if (dist(0)<10 || dist(1)<10) {
      turn(TURN180 - 100, dir);
      dir = !dir;
    } else {
      strait();
    }
   }
  Serial.flush();
  Serial.println("ok");
}

#define BFS_STEP 500UL
void bfs_sweep() {
  int stage = 0;
  unsigned long last = millis();
  while (!Serial.available()) {
    stage++;
    m.feed(0, 127);
    m.feed(1, 127);
    while (!Serial.available()) {
      if (millis()-last > stage*BFS_STEP) {
        turn(TURN90, 0);
        last = millis();
        break;
      }
    }
    m.feed(0, 127);
    m.feed(1, 127);
    while (!Serial.available()) {
      if (millis()-last > stage*BFS_STEP) {
        turn(TURN90, 0);
        last = millis();
        break;
      }
    }
  }
  Serial.flush();
  Serial.println("ok");
}
